@@ -18,6 +18,8 @@ from ipaddress import (
 from json import dumps, loads
 from uuid import UUID
 
+from dateutil.parser import parse
+
 from pg8000.exceptions import InterfaceError
 
 
@@ -185,6 +187,10 @@ def numeric_out(d):
     return str(d)
 
 
+def point_in(data):
+    return tuple(map(float, data[1:-1].split(",")))
+
+
 def pg_interval_in(data):
     return PGInterval.from_str(data)
 
@@ -214,13 +220,22 @@ def timestamp_in(data):
     if data in ("infinity", "-infinity"):
         return data
 
-    pattern = "%Y-%m-%d %H:%M:%S.%f" if "." in data else "%Y-%m-%d %H:%M:%S"
-    return Datetime.strptime(data, pattern)
+    try:
+        pattern = "%Y-%m-%d %H:%M:%S.%f" if "." in data else "%Y-%m-%d %H:%M:%S"
+        return Datetime.strptime(data, pattern)
+    except ValueError:
+        return parse(data)
 
 
 def timestamptz_in(data):
-    patt = "%Y-%m-%d %H:%M:%S.%f%z" if "." in data else "%Y-%m-%d %H:%M:%S%z"
-    return Datetime.strptime(data + "00", patt)
+    if data in ("infinity", "-infinity"):
+        return data
+
+    try:
+        patt = "%Y-%m-%d %H:%M:%S.%f%z" if "." in data else "%Y-%m-%d %H:%M:%S%z"
+        return Datetime.strptime(f"{data}00", patt)
+    except ValueError:
+        return parse(data)
 
 
 def unknown_out(v):
@@ -242,7 +257,7 @@ def uuid_in(data):
 class PGInterval:
     UNIT_MAP = {
         "millennia": "millennia",
-        "millenium": "millennia",
+        "millennium": "millennia",
         "centuries": "centuries",
         "century": "centuries",
         "decades": "decades",
@@ -594,7 +609,8 @@ def array_string_escape(v):
     if (
         len(val) == 0
         or val == "NULL"
-        or any([c in val for c in ("{", "}", ",", " ", "\\")])
+        or any(c.isspace() for c in val)
+        or any(c in val for c in ("{", "}", ",", "\\"))
     ):
         val = f'"{val}"'
     return val
@@ -603,10 +619,12 @@ def array_string_escape(v):
 def array_out(ar):
     result = []
     for v in ar:
-
-        if isinstance(v, (list, tuple)):
+        if isinstance(v, list):
             val = array_out(v)
 
+        elif isinstance(v, tuple):
+            val = f'"{composite_out(v)}"'
+
         elif v is None:
             val = "NULL"
 
@@ -624,7 +642,36 @@ def array_out(ar):
 
         result.append(val)
 
-    return "{" + ",".join(result) + "}"
+    return f'{{{",".join(result)}}}'
+
+
+def composite_out(ar):
+    result = []
+    for v in ar:
+        if isinstance(v, list):
+            val = array_out(v)
+
+        elif isinstance(v, tuple):
+            val = composite_out(v)
+
+        elif v is None:
+            val = ""
+
+        elif isinstance(v, dict):
+            val = array_string_escape(json_out(v))
+
+        elif isinstance(v, (bytes, bytearray)):
+            val = f'"\\{bytes_out(v)}"'
+
+        elif isinstance(v, str):
+            val = array_string_escape(v)
+
+        else:
+            val = make_param(PY_TYPES, v)
+
+        result.append(val)
+
+    return f'({",".join(result)})'
 
 
 PY_PG = {
@@ -670,7 +717,7 @@ PY_TYPES = {
     str: string_out,  # unknown
     int: int_out,
     list: array_out,
-    tuple: array_out,
+    tuple: composite_out,
 }
 
 
@@ -706,6 +753,7 @@ PG_TYPES = {
     NUMERIC: numeric_in,  # numeric
     NUMERIC_ARRAY: numeric_array_in,  # numeric[]
     OID: int,  # oid
+    POINT: point_in,  # point
     INTERVAL: interval_in,  # interval
     INTERVAL_ARRAY: interval_array_in,  # interval[]
     REAL: float,  # float4
