@@ -1,4 +1,8 @@
-from datetime import date as Date, datetime as Datetime, time as Time
+from datetime import (
+    date as Date,
+    datetime as Datetime,
+    time as Time,
+)
 from itertools import count, islice
 from time import localtime
 from warnings import warn
@@ -28,6 +32,7 @@ from pg8000.converters import (
     NUMERIC_ARRAY,
     OID,
     PGInterval,
+    PY_PG,
     STRING,
     TEXT,
     TEXT_ARRAY,
@@ -109,7 +114,7 @@ BINARY = bytes
 
 
 def PgDate(year, month, day):
-    """Constuct an object holding a date value.
+    """Construct an object holding a date value.
 
     This function is part of the `DBAPI 2.0 specification
     <http://www.python.org/dev/peps/pep-0249/>`_.
@@ -154,7 +159,7 @@ def DateFromTicks(ticks):
 
 
 def TimeFromTicks(ticks):
-    """Construct an objet holding a time value from the given ticks value
+    """Construct an object holding a time value from the given ticks value
     (number of seconds since the epoch).
 
     This function is part of the `DBAPI 2.0 specification
@@ -201,7 +206,6 @@ def connect(
     application_name=None,
     replication=None,
 ):
-
     return Connection(
         user,
         host=host,
@@ -506,7 +510,6 @@ class Cursor:
         operation = f"CALL {procname}(" + ", ".join(["%s" for _ in args]) + ")"
 
         try:
-
             statement, vals = convert_paramstyle("format", operation, args)
 
             self._context = self._c.execute_unnamed(statement, vals=vals)
@@ -618,7 +621,7 @@ class Cursor:
                 oid = size
             else:
                 try:
-                    oid, _ = self._c.py_types[size]
+                    oid = PY_PG[size]
                 except KeyError:
                     oid = UNKNOWN
             oids.append(oid)
@@ -634,6 +637,9 @@ class Cursor:
 
 
 class Connection(CoreConnection):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.autocommit = False
 
     # DBAPI Extension: supply exceptions as attributes on the connection
     Warning = property(lambda self: self._getError(Warning))
